@@ -74,7 +74,7 @@ typedef struct _MetaWindowMenu MetaWindowMenu;
 typedef void (* MetaWindowMenuFunc) (MetaWindowMenu *menu,
                                      Display        *xdisplay,
                                      Window          client_xwindow,
-				     Time	     timestamp,
+                                     guint32         timestamp,
                                      MetaMenuOp      op,
                                      int             workspace,
                                      gpointer        data);
@@ -143,8 +143,7 @@ typedef enum
   META_CURSOR_SW_RESIZE,
   META_CURSOR_NE_RESIZE,
   META_CURSOR_NW_RESIZE,
-  META_CURSOR_MOVE_WINDOW,
-  META_CURSOR_RESIZE_WINDOW,
+  META_CURSOR_MOVE_OR_RESIZE_WINDOW,
   META_CURSOR_BUSY
 
 } MetaCursor;
@@ -201,7 +200,6 @@ typedef enum
 
 /* Relative directions or sides seem to come up all over the place... */
 /* FIXME: Replace
- *   place.[ch]:MetaWindowEdgePosition,
  *   screen.[ch]:MetaScreenDirection,
  *   workspace.[ch]:MetaMotionDirection,
  * with the use of MetaDirection.
