@@ -56,9 +56,6 @@ find_largest_sizes (gulong *data,
   while (nitems > 0)
     {
       int w, h;
-      gboolean replace;
-
-      replace = FALSE;
 
       if (nitems < 3)
         return FALSE; /* no space for w, h */
@@ -66,7 +63,7 @@ find_largest_sizes (gulong *data,
       w = data[0];
       h = data[1];
 
-      if (nitems < ((w * h) + 2))
+      if (nitems < ((gulong)(w * h) + 2))
         return FALSE; /* not enough data */
 
       *width = MAX (w, *width);
@@ -122,7 +119,7 @@ find_best_size (gulong  *data,
       w = data[0];
       h = data[1];
 
-      if (nitems < ((w * h) + 2))
+      if (nitems < ((gulong)(w * h) + 2))
         break; /* not enough data */
 
       if (best_start == NULL)
