@@ -196,42 +196,6 @@ meta_stack_thaw (MetaStack *stack)
   meta_stack_sync_to_server (stack);
 }
 
-static gboolean
-window_is_fullscreen_size (MetaWindow *window)
-{
-  int i;
-
-  if (meta_rectangle_could_fit_rect (&window->rect, &window->screen->rect))
-    {
-      /* we use the work area since windows that try to
-       * position at 0,0 will get pushed down by menu panel
-       */
-      MetaRectangle workarea;
-      
-      meta_window_get_work_area_current_xinerama (window, &workarea);
-      if (meta_rectangle_contains_rect (&window->rect, &workarea))
-        return TRUE;
-    }
-  
-  i = 0;
-  while (i < window->screen->n_xinerama_infos)
-    {
-      if (meta_rectangle_could_fit_rect (&window->rect,
-                                         &window->screen->xinerama_infos[i].rect))
-        {
-          MetaRectangle workarea;
-          
-          meta_window_get_work_area_current_xinerama (window, &workarea);
-          if (meta_rectangle_contains_rect (&window->rect, &workarea))
-            return TRUE;
-        }
-      
-      ++i;
-    }
-
-  return FALSE;
-}
-
 static gboolean
 is_focused_foreach (MetaWindow *window,
                     void       *data)
@@ -283,7 +247,7 @@ get_standalone_layer (MetaWindow *window)
 
       if (window->wm_state_below)
         layer = META_LAYER_BOTTOM;
-      else if ((window->fullscreen || window_is_fullscreen_size (window)) &&
+      else if (window->fullscreen &&
                (focused_transient ||
                 window == window->display->expected_focus_window ||
                 window->display->expected_focus_window == NULL ||
@@ -1196,14 +1160,14 @@ meta_stack_sync_to_server (MetaStack *stack)
                    stack->screen->display->atom_net_client_list,
                    XA_WINDOW,
                    32, PropModeReplace,
-                   stack->windows->data,
+                   (unsigned char *)stack->windows->data,
                    stack->windows->len);
   XChangeProperty (stack->screen->display->xdisplay,
                    stack->screen->xroot,
                    stack->screen->display->atom_net_client_list_stacking,
                    XA_WINDOW,
                    32, PropModeReplace,
-                   stacked->data,
+                   (unsigned char *)stacked->data,
                    stacked->len);
 
   g_array_free (stacked, TRUE);
